package net.knarcraft.blacksmith.manager;

import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * A manager for keeping track of players using scrappers
 */
public class PlayerUsageManager {

    private static final Map<Player, Set<Long>> playerUsages = new HashMap<>();

    /**
     * Register that a player has used a scrapper
     *
     * @param player <p>The player using a scrapper</p>
     */
    public static void registerUsage(@NotNull Player player) {
        playerUsages.putIfAbsent(player, new HashSet<>());
        Set<Long> usages = playerUsages.get(player);
        usages.add(System.currentTimeMillis());
    }

    /**
     * Gets how many times the given player has used a scrapper after the given timestamp
     *
     * @param player         <p>The player to check usages for</p>
     * @param afterTimestamp <p>The timestamp of when to start looking for usages</p>
     * @return <p>The number of scrapper uses for the given player</p>
     */
    public static int getUsages(@NotNull Player player, @NotNull Long afterTimestamp) {
        Set<Long> usages = playerUsages.get(player);
        if (usages == null) {
            return 0;
        }

        int uses = 0;
        Set<Long> expired = new HashSet<>();
        for (Long usageTime : usages) {
            if (usageTime < afterTimestamp) {
                expired.add(usageTime);
            } else {
                uses++;
            }
        }

        // Remove expired data
        usages.removeAll(expired);

        return uses;
    }

    /**
     * Removes all expired timestamps
     *
     * @param afterTimestamp <p>The timestamp for the earliest time of valid timestamps</p>
     */
    public static void removeExpiredData(@NotNull Long afterTimestamp) {
        for (Set<Long> usageSet : playerUsages.values()) {
            Set<Long> expired = new HashSet<>();
            for (Long item : usageSet) {
                if (item < afterTimestamp) {
                    expired.add(item);
                }
            }
            usageSet.removeAll(expired);
        }
    }

}
